/*
 * <lalVerbatim file="SendRecvCV">
 * $Id: SendRecv.m4,v 1.2 2001/08/21 04:19:49 jolien Exp $
 * </lalVerbatim>
 */

/*
 * <lalLaTeX>
 *
 * \subsection{Module \texttt{SendRecv.c}}
 *
 * Routines to perform basic MPI sending and receiving of LAL data types.
 *
 * \subsubsection*{Prototypes}
 * \vspace{0.1in}
 * \input{SendRecvCP}
 * \idx{LALMPISendMsg()}
 * \idx{LALMPIRecvMsg()}
 * \idx{LALMPISendCHAR()}
 * \idx{LALMPIRecvCHAR()}
 * \idx{LALMPISendINT2()}
 * \idx{LALMPIRecvINT2()}
 * \idx{LALMPISendINT4()}
 * \idx{LALMPIRecvINT4()}
 * \idx{LALMPISendINT8()}
 * \idx{LALMPIRecvINT8()}
 * \idx{LALMPISendUINT2()}
 * \idx{LALMPIRecvUINT2()}
 * \idx{LALMPISendUINT4()}
 * \idx{LALMPIRecvUINT4()}
 * \idx{LALMPISendUINT8()}
 * \idx{LALMPIRecvUINT8()}
 * \idx{LALMPISendREAL4()}
 * \idx{LALMPIRecvREAL4()}
 * \idx{LALMPISendREAL8()}
 * \idx{LALMPIRecvREAL8()}
 * \idx{LALMPISendCOMPLEX8()}
 * \idx{LALMPIRecvCOMPLEX8()}
 * \idx{LALMPISendCOMPLEX16()}
 * \idx{LALMPIRecvCOMPLEX16()}
 * \idx{LALMPISendCHARVector()}
 * \idx{LALMPIRecvCHARVector()}
 * \idx{LALMPISendINT2Vector()}
 * \idx{LALMPIRecvINT2Vector()}
 * \idx{LALMPISendINT4Vector()}
 * \idx{LALMPIRecvINT4Vector()}
 * \idx{LALMPISendINT8Vector()}
 * \idx{LALMPIRecvINT8Vector()}
 * \idx{LALMPISendUINT2Vector()}
 * \idx{LALMPIRecvUINT2Vector()}
 * \idx{LALMPISendUINT4Vector()}
 * \idx{LALMPIRecvUINT4Vector()}
 * \idx{LALMPISendUINT8Vector()}
 * \idx{LALMPIRecvUINT8Vector()}
 * \idx{LALMPISendREAL4Vector()}
 * \idx{LALMPIRecvREAL4Vector()}
 * \idx{LALMPISendREAL8Vector()}
 * \idx{LALMPIRecvREAL8Vector()}
 * \idx{LALMPISendCOMPLEX8Vector()}
 * \idx{LALMPIRecvCOMPLEX8Vector()}
 * \idx{LALMPISendCOMPLEX16Vector()}
 * \idx{LALMPIRecvCOMPLEX16Vector()}
 * \idx{LALMPISendINT2TimeSeries()}
 * \idx{LALMPIRecvINT2TimeSeries()}
 * \idx{LALMPISendINT4TimeSeries()}
 * \idx{LALMPIRecvINT4TimeSeries()}
 * \idx{LALMPISendINT8TimeSeries()}
 * \idx{LALMPIRecvINT8TimeSeries()}
 * \idx{LALMPISendUINT2TimeSeries()}
 * \idx{LALMPIRecvUINT2TimeSeries()}
 * \idx{LALMPISendUINT4TimeSeries()}
 * \idx{LALMPIRecvUINT4TimeSeries()}
 * \idx{LALMPISendUINT8TimeSeries()}
 * \idx{LALMPIRecvUINT8TimeSeries()}
 * \idx{LALMPISendREAL4TimeSeries()}
 * \idx{LALMPIRecvREAL4TimeSeries()}
 * \idx{LALMPISendREAL8TimeSeries()}
 * \idx{LALMPIRecvREAL8TimeSeries()}
 * \idx{LALMPISendCOMPLEX8TimeSeries()}
 * \idx{LALMPIRecvCOMPLEX8TimeSeries()}
 * \idx{LALMPISendCOMPLEX16TimeSeries()}
 * \idx{LALMPIRecvCOMPLEX16TimeSeries()}
 * \idx{LALMPISendINT2FrequencySeries()}
 * \idx{LALMPIRecvINT2FrequencySeries()}
 * \idx{LALMPISendINT4FrequencySeries()}
 * \idx{LALMPIRecvINT4FrequencySeries()}
 * \idx{LALMPISendINT8FrequencySeries()}
 * \idx{LALMPIRecvINT8FrequencySeries()}
 * \idx{LALMPISendUINT2FrequencySeries()}
 * \idx{LALMPIRecvUINT2FrequencySeries()}
 * \idx{LALMPISendUINT4FrequencySeries()}
 * \idx{LALMPIRecvUINT4FrequencySeries()}
 * \idx{LALMPISendUINT8FrequencySeries()}
 * \idx{LALMPIRecvUINT8FrequencySeries()}
 * \idx{LALMPISendREAL4FrequencySeries()}
 * \idx{LALMPIRecvREAL4FrequencySeries()}
 * \idx{LALMPISendREAL8FrequencySeries()}
 * \idx{LALMPIRecvREAL8FrequencySeries()}
 * \idx{LALMPISendCOMPLEX8FrequencySeries()}
 * \idx{LALMPIRecvCOMPLEX8FrequencySeries()}
 * \idx{LALMPISendCOMPLEX16FrequencySeries()}
 * \idx{LALMPIRecvCOMPLEX16FrequencySeries()}
 *
 * \subsubsection*{Description}
 *
 * The routines \verb+LALMPISend+$\langle\textit{type}\rangle$ and
 * \verb+LALMPIRecv+$\langle\textit{type}\rangle$ must be used in pairs:
 * the sender must specify the MPI process number of the receiving process
 * as the input variable \verb+dest+, while the recieving process must specify
 * the MPI process number of the sending process as its input variable
 * \verb+source+.  The object that is sent/recived is the output for both
 * classes of functions.  The parameter is the MPI communicator \verb+comm+.
 *
 * The exception is the function \verb+LALMPIRecvMsg()+.  This function does
 * not specify a source---it receives from any process.  The function
 * \verb+LALMPISendMsg()+ is used to inform the reciever, which should be
 * listening with \verb+LALMPIRecvMsg()+, that the sender wishes to exchange
 * information.
 *
 * </lalLaTeX>
 */

#if 0 /* autodoc block */
<lalLaTeX>
The following is a simple example of a commissar-comrad situation.
 
\begin{verbatim}
#include <stdio.h>
#include <math.h>
#include <mpi.h>
#include <lal/LALStdlib.h>
#include <lal/Comm.h>

INT4 lalDebugLevel = 0;

int Commissar( INT4 size, MPI_Comm comm );
int Comrad( INT4 size, INT4 rank, MPI_Comm comm );

int main( int argc, char *argv[] )
{
  int size, rank, code;

  MPI_Init( &argc, &argv );
  MPI_Comm_size( MPI_COMM_WORLD, &size );
  MPI_Comm_rank( MPI_COMM_WORLD, &rank );

  code = rank ? Comrad( size, rank, MPI_COMM_WORLD )
    : Commissar( size, MPI_COMM_WORLD );

  MPI_Finalize();
  return code;
}

int Commissar( INT4 size, MPI_Comm comm )
{
  static LALStatus status;
  MPIMessage msg;
  REAL4 data;

  while ( size > 1 ) /* while there are still comrads */
  {
    /* receive order from comrad */
    LALMPIRecvMsg( &status, &msg, comm );

    /* handle the message code */
    switch ( msg.msg )
    {
      case 0: /* comrad is quitting */
        --size;
        break;

      case 1: /* comrad is sending data */
        LALMPIRecvREAL4( &status, &data, msg.source, comm );
        printf( "data from comrad %d: %f", msg.source, data );
        break;

      default: /* should never happen */
        return 1;
    }
  }

  return 0;
}

int Comrad( INT4 size, INT4 rank, MPI_Comm comm )
{
  static LALStatus status;
  MPIMessage msg;
  REAL4 data = fmod( rank, size ); /* some junk data */

  /* send message to commissar indicating that comrad is ready to send data */
  msg.msg    = 1;    /* indicates that data is to be communicated           */
  msg.send   = 1;    /* indicates that comrad (local process) will send     */
  msg.source = rank; /* rank of local process                               */
  LALMPISendMsg( &status, &msg, 0, comm ); /* rank zero is commissar */

  /* commissar now knows data is coming so send it */
  LALMPISendREAL4( &status, &data, 0, comm );

  /* send message to commissar indicating that comrad is quitting */
  msg.msg    = 0;    /* indicates that comrad is quitting         */
  msg.source = rank; /* rank of local process                     */
  LALMPISendMsg( &status, &msg, 0, comm );

  return 0;
}
\end{verbatim}
 
</lalLaTeX>
#endif /* autodoc block */

/*
 * <lalLaTeX>
 * \vfill{\footnotesize\input{SendRecvCV}}
 * </lalLaTeX>
 */

#include <lal/LALStdlib.h>
#include <lal/Comm.h>

NRCSID( SENDRECVC, "$Id: SendRecv.m4,v 1.2 2001/08/21 04:19:49 jolien Exp $" );


/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendMsg( LALStatus *status, MPIMessage *msg, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendMsg", SENDRECVC );

  ASSERT( msg, status, COMMH_ENULL, COMMH_MSGENULL );

  code = MPI_Send( msg, sizeof( *msg ), MPI_BYTE, dest, MPIMsg, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}


/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvMsg( LALStatus *status, MPIMessage *msg, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  INT4       code;

  INITSTATUS( status, "LALMPIRecvMsg", SENDRECVC );

  ASSERT( msg, status, COMMH_ENULL, COMMH_MSGENULL );

  code = MPI_Recv( msg, sizeof( *msg ), MPI_BYTE, MPI_ANY_SOURCE, MPIMsg,
                   comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}




















/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendCOMPLEX16 ( LALStatus *status, COMPLEX16 *element, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendCOMPLEX16", SENDRECVC );

  ASSERT( element, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the element */
  code = MPI_Send( element, sizeof( *element ), MPI_BYTE, dest, MPIZ, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvCOMPLEX16 ( LALStatus *status, COMPLEX16 *element, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvCOMPLEX16", SENDRECVC );

  ASSERT( element, status, COMMH_ENULL, COMMH_MSGENULL );

  code = MPI_Recv( element, sizeof( *element ), MPI_BYTE, source, MPIZ, comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}




















/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendCOMPLEX8 ( LALStatus *status, COMPLEX8 *element, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendCOMPLEX8", SENDRECVC );

  ASSERT( element, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the element */
  code = MPI_Send( element, sizeof( *element ), MPI_BYTE, dest, MPIC, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvCOMPLEX8 ( LALStatus *status, COMPLEX8 *element, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvCOMPLEX8", SENDRECVC );

  ASSERT( element, status, COMMH_ENULL, COMMH_MSGENULL );

  code = MPI_Recv( element, sizeof( *element ), MPI_BYTE, source, MPIC, comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}




















/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendREAL8 ( LALStatus *status, REAL8 *element, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendREAL8", SENDRECVC );

  ASSERT( element, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the element */
  code = MPI_Send( element, sizeof( *element ), MPI_BYTE, dest, MPID, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvREAL8 ( LALStatus *status, REAL8 *element, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvREAL8", SENDRECVC );

  ASSERT( element, status, COMMH_ENULL, COMMH_MSGENULL );

  code = MPI_Recv( element, sizeof( *element ), MPI_BYTE, source, MPID, comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}




















/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendREAL4 ( LALStatus *status, REAL4 *element, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendREAL4", SENDRECVC );

  ASSERT( element, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the element */
  code = MPI_Send( element, sizeof( *element ), MPI_BYTE, dest, MPIS, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvREAL4 ( LALStatus *status, REAL4 *element, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvREAL4", SENDRECVC );

  ASSERT( element, status, COMMH_ENULL, COMMH_MSGENULL );

  code = MPI_Recv( element, sizeof( *element ), MPI_BYTE, source, MPIS, comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}




















/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendINT2 ( LALStatus *status, INT2 *element, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendINT2", SENDRECVC );

  ASSERT( element, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the element */
  code = MPI_Send( element, sizeof( *element ), MPI_BYTE, dest, MPII2, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvINT2 ( LALStatus *status, INT2 *element, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvINT2", SENDRECVC );

  ASSERT( element, status, COMMH_ENULL, COMMH_MSGENULL );

  code = MPI_Recv( element, sizeof( *element ), MPI_BYTE, source, MPII2, comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}




















/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendINT4 ( LALStatus *status, INT4 *element, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendINT4", SENDRECVC );

  ASSERT( element, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the element */
  code = MPI_Send( element, sizeof( *element ), MPI_BYTE, dest, MPII4, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvINT4 ( LALStatus *status, INT4 *element, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvINT4", SENDRECVC );

  ASSERT( element, status, COMMH_ENULL, COMMH_MSGENULL );

  code = MPI_Recv( element, sizeof( *element ), MPI_BYTE, source, MPII4, comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}




















/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendINT8 ( LALStatus *status, INT8 *element, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendINT8", SENDRECVC );

  ASSERT( element, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the element */
  code = MPI_Send( element, sizeof( *element ), MPI_BYTE, dest, MPII8, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvINT8 ( LALStatus *status, INT8 *element, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvINT8", SENDRECVC );

  ASSERT( element, status, COMMH_ENULL, COMMH_MSGENULL );

  code = MPI_Recv( element, sizeof( *element ), MPI_BYTE, source, MPII8, comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}




















/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendUINT2 ( LALStatus *status, UINT2 *element, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendUINT2", SENDRECVC );

  ASSERT( element, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the element */
  code = MPI_Send( element, sizeof( *element ), MPI_BYTE, dest, MPIU2, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvUINT2 ( LALStatus *status, UINT2 *element, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvUINT2", SENDRECVC );

  ASSERT( element, status, COMMH_ENULL, COMMH_MSGENULL );

  code = MPI_Recv( element, sizeof( *element ), MPI_BYTE, source, MPIU2, comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}




















/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendUINT4 ( LALStatus *status, UINT4 *element, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendUINT4", SENDRECVC );

  ASSERT( element, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the element */
  code = MPI_Send( element, sizeof( *element ), MPI_BYTE, dest, MPIU4, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvUINT4 ( LALStatus *status, UINT4 *element, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvUINT4", SENDRECVC );

  ASSERT( element, status, COMMH_ENULL, COMMH_MSGENULL );

  code = MPI_Recv( element, sizeof( *element ), MPI_BYTE, source, MPIU4, comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}




















/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendUINT8 ( LALStatus *status, UINT8 *element, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendUINT8", SENDRECVC );

  ASSERT( element, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the element */
  code = MPI_Send( element, sizeof( *element ), MPI_BYTE, dest, MPIU8, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvUINT8 ( LALStatus *status, UINT8 *element, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvUINT8", SENDRECVC );

  ASSERT( element, status, COMMH_ENULL, COMMH_MSGENULL );

  code = MPI_Recv( element, sizeof( *element ), MPI_BYTE, source, MPIU8, comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}




















/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendCHAR ( LALStatus *status, CHAR *element, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendCHAR", SENDRECVC );

  ASSERT( element, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the element */
  code = MPI_Send( element, sizeof( *element ), MPI_BYTE, dest, MPICHAR, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvCHAR ( LALStatus *status, CHAR *element, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvCHAR", SENDRECVC );

  ASSERT( element, status, COMMH_ENULL, COMMH_MSGENULL );

  code = MPI_Recv( element, sizeof( *element ), MPI_BYTE, source, MPICHAR, comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}























/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendCOMPLEX16Vector ( LALStatus *status, COMPLEX16Vector *vector, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendCOMPLEX16Vector", SENDRECVC );

  ASSERT( vector,         status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->data,   status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->length, status, COMMH_ESIZE, COMMH_MSGESIZE );

  /* send the vector structure */
  code = MPI_Send( vector, sizeof( *vector ), MPI_BYTE, dest, MPIZVector, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send the vector data */
  code = MPI_Send( vector->data, vector->length * sizeof( *vector->data ),
      MPI_BYTE, dest, MPIZVectorData, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvCOMPLEX16Vector ( LALStatus *status, COMPLEX16Vector *vector, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  COMPLEX16Vector vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvCOMPLEX16Vector", SENDRECVC );

  ASSERT( vector,         status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->data,   status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->length, status, COMMH_ESIZE, COMMH_MSGESIZE );

  /* receive temporary vector structure to check */
  code = MPI_Recv( &vtmp, sizeof( vtmp ), MPI_BYTE, source, MPIZVector, comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* make sure that the vector lengths agree */
  if ( vector->length != vtmp.length )
  {
    ABORT( status, COMMH_ESZMM, COMMH_MSGESZMM );
  }

  /* receive the vector data */
  code = MPI_Recv( vector->data, vector->length * sizeof( *vector->data ),
      MPI_BYTE, source, MPIZVectorData, comm, &stat );

  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}






















/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendCOMPLEX8Vector ( LALStatus *status, COMPLEX8Vector *vector, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendCOMPLEX8Vector", SENDRECVC );

  ASSERT( vector,         status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->data,   status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->length, status, COMMH_ESIZE, COMMH_MSGESIZE );

  /* send the vector structure */
  code = MPI_Send( vector, sizeof( *vector ), MPI_BYTE, dest, MPICVector, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send the vector data */
  code = MPI_Send( vector->data, vector->length * sizeof( *vector->data ),
      MPI_BYTE, dest, MPICVectorData, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvCOMPLEX8Vector ( LALStatus *status, COMPLEX8Vector *vector, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  COMPLEX8Vector vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvCOMPLEX8Vector", SENDRECVC );

  ASSERT( vector,         status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->data,   status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->length, status, COMMH_ESIZE, COMMH_MSGESIZE );

  /* receive temporary vector structure to check */
  code = MPI_Recv( &vtmp, sizeof( vtmp ), MPI_BYTE, source, MPICVector, comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* make sure that the vector lengths agree */
  if ( vector->length != vtmp.length )
  {
    ABORT( status, COMMH_ESZMM, COMMH_MSGESZMM );
  }

  /* receive the vector data */
  code = MPI_Recv( vector->data, vector->length * sizeof( *vector->data ),
      MPI_BYTE, source, MPICVectorData, comm, &stat );

  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}






















/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendREAL8Vector ( LALStatus *status, REAL8Vector *vector, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendREAL8Vector", SENDRECVC );

  ASSERT( vector,         status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->data,   status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->length, status, COMMH_ESIZE, COMMH_MSGESIZE );

  /* send the vector structure */
  code = MPI_Send( vector, sizeof( *vector ), MPI_BYTE, dest, MPIDVector, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send the vector data */
  code = MPI_Send( vector->data, vector->length * sizeof( *vector->data ),
      MPI_BYTE, dest, MPIDVectorData, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvREAL8Vector ( LALStatus *status, REAL8Vector *vector, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  REAL8Vector vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvREAL8Vector", SENDRECVC );

  ASSERT( vector,         status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->data,   status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->length, status, COMMH_ESIZE, COMMH_MSGESIZE );

  /* receive temporary vector structure to check */
  code = MPI_Recv( &vtmp, sizeof( vtmp ), MPI_BYTE, source, MPIDVector, comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* make sure that the vector lengths agree */
  if ( vector->length != vtmp.length )
  {
    ABORT( status, COMMH_ESZMM, COMMH_MSGESZMM );
  }

  /* receive the vector data */
  code = MPI_Recv( vector->data, vector->length * sizeof( *vector->data ),
      MPI_BYTE, source, MPIDVectorData, comm, &stat );

  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}






















/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendREAL4Vector ( LALStatus *status, REAL4Vector *vector, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendREAL4Vector", SENDRECVC );

  ASSERT( vector,         status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->data,   status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->length, status, COMMH_ESIZE, COMMH_MSGESIZE );

  /* send the vector structure */
  code = MPI_Send( vector, sizeof( *vector ), MPI_BYTE, dest, MPISVector, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send the vector data */
  code = MPI_Send( vector->data, vector->length * sizeof( *vector->data ),
      MPI_BYTE, dest, MPISVectorData, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvREAL4Vector ( LALStatus *status, REAL4Vector *vector, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  REAL4Vector vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvREAL4Vector", SENDRECVC );

  ASSERT( vector,         status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->data,   status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->length, status, COMMH_ESIZE, COMMH_MSGESIZE );

  /* receive temporary vector structure to check */
  code = MPI_Recv( &vtmp, sizeof( vtmp ), MPI_BYTE, source, MPISVector, comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* make sure that the vector lengths agree */
  if ( vector->length != vtmp.length )
  {
    ABORT( status, COMMH_ESZMM, COMMH_MSGESZMM );
  }

  /* receive the vector data */
  code = MPI_Recv( vector->data, vector->length * sizeof( *vector->data ),
      MPI_BYTE, source, MPISVectorData, comm, &stat );

  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}






















/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendINT2Vector ( LALStatus *status, INT2Vector *vector, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendINT2Vector", SENDRECVC );

  ASSERT( vector,         status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->data,   status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->length, status, COMMH_ESIZE, COMMH_MSGESIZE );

  /* send the vector structure */
  code = MPI_Send( vector, sizeof( *vector ), MPI_BYTE, dest, MPII2Vector, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send the vector data */
  code = MPI_Send( vector->data, vector->length * sizeof( *vector->data ),
      MPI_BYTE, dest, MPII2VectorData, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvINT2Vector ( LALStatus *status, INT2Vector *vector, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  INT2Vector vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvINT2Vector", SENDRECVC );

  ASSERT( vector,         status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->data,   status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->length, status, COMMH_ESIZE, COMMH_MSGESIZE );

  /* receive temporary vector structure to check */
  code = MPI_Recv( &vtmp, sizeof( vtmp ), MPI_BYTE, source, MPII2Vector, comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* make sure that the vector lengths agree */
  if ( vector->length != vtmp.length )
  {
    ABORT( status, COMMH_ESZMM, COMMH_MSGESZMM );
  }

  /* receive the vector data */
  code = MPI_Recv( vector->data, vector->length * sizeof( *vector->data ),
      MPI_BYTE, source, MPII2VectorData, comm, &stat );

  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}






















/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendINT4Vector ( LALStatus *status, INT4Vector *vector, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendINT4Vector", SENDRECVC );

  ASSERT( vector,         status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->data,   status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->length, status, COMMH_ESIZE, COMMH_MSGESIZE );

  /* send the vector structure */
  code = MPI_Send( vector, sizeof( *vector ), MPI_BYTE, dest, MPII4Vector, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send the vector data */
  code = MPI_Send( vector->data, vector->length * sizeof( *vector->data ),
      MPI_BYTE, dest, MPII4VectorData, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvINT4Vector ( LALStatus *status, INT4Vector *vector, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  INT4Vector vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvINT4Vector", SENDRECVC );

  ASSERT( vector,         status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->data,   status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->length, status, COMMH_ESIZE, COMMH_MSGESIZE );

  /* receive temporary vector structure to check */
  code = MPI_Recv( &vtmp, sizeof( vtmp ), MPI_BYTE, source, MPII4Vector, comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* make sure that the vector lengths agree */
  if ( vector->length != vtmp.length )
  {
    ABORT( status, COMMH_ESZMM, COMMH_MSGESZMM );
  }

  /* receive the vector data */
  code = MPI_Recv( vector->data, vector->length * sizeof( *vector->data ),
      MPI_BYTE, source, MPII4VectorData, comm, &stat );

  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}






















/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendINT8Vector ( LALStatus *status, INT8Vector *vector, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendINT8Vector", SENDRECVC );

  ASSERT( vector,         status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->data,   status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->length, status, COMMH_ESIZE, COMMH_MSGESIZE );

  /* send the vector structure */
  code = MPI_Send( vector, sizeof( *vector ), MPI_BYTE, dest, MPII8Vector, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send the vector data */
  code = MPI_Send( vector->data, vector->length * sizeof( *vector->data ),
      MPI_BYTE, dest, MPII8VectorData, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvINT8Vector ( LALStatus *status, INT8Vector *vector, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  INT8Vector vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvINT8Vector", SENDRECVC );

  ASSERT( vector,         status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->data,   status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->length, status, COMMH_ESIZE, COMMH_MSGESIZE );

  /* receive temporary vector structure to check */
  code = MPI_Recv( &vtmp, sizeof( vtmp ), MPI_BYTE, source, MPII8Vector, comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* make sure that the vector lengths agree */
  if ( vector->length != vtmp.length )
  {
    ABORT( status, COMMH_ESZMM, COMMH_MSGESZMM );
  }

  /* receive the vector data */
  code = MPI_Recv( vector->data, vector->length * sizeof( *vector->data ),
      MPI_BYTE, source, MPII8VectorData, comm, &stat );

  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}






















/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendUINT2Vector ( LALStatus *status, UINT2Vector *vector, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendUINT2Vector", SENDRECVC );

  ASSERT( vector,         status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->data,   status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->length, status, COMMH_ESIZE, COMMH_MSGESIZE );

  /* send the vector structure */
  code = MPI_Send( vector, sizeof( *vector ), MPI_BYTE, dest, MPIU2Vector, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send the vector data */
  code = MPI_Send( vector->data, vector->length * sizeof( *vector->data ),
      MPI_BYTE, dest, MPIU2VectorData, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvUINT2Vector ( LALStatus *status, UINT2Vector *vector, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  UINT2Vector vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvUINT2Vector", SENDRECVC );

  ASSERT( vector,         status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->data,   status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->length, status, COMMH_ESIZE, COMMH_MSGESIZE );

  /* receive temporary vector structure to check */
  code = MPI_Recv( &vtmp, sizeof( vtmp ), MPI_BYTE, source, MPIU2Vector, comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* make sure that the vector lengths agree */
  if ( vector->length != vtmp.length )
  {
    ABORT( status, COMMH_ESZMM, COMMH_MSGESZMM );
  }

  /* receive the vector data */
  code = MPI_Recv( vector->data, vector->length * sizeof( *vector->data ),
      MPI_BYTE, source, MPIU2VectorData, comm, &stat );

  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}






















/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendUINT4Vector ( LALStatus *status, UINT4Vector *vector, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendUINT4Vector", SENDRECVC );

  ASSERT( vector,         status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->data,   status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->length, status, COMMH_ESIZE, COMMH_MSGESIZE );

  /* send the vector structure */
  code = MPI_Send( vector, sizeof( *vector ), MPI_BYTE, dest, MPIU4Vector, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send the vector data */
  code = MPI_Send( vector->data, vector->length * sizeof( *vector->data ),
      MPI_BYTE, dest, MPIU4VectorData, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvUINT4Vector ( LALStatus *status, UINT4Vector *vector, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  UINT4Vector vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvUINT4Vector", SENDRECVC );

  ASSERT( vector,         status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->data,   status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->length, status, COMMH_ESIZE, COMMH_MSGESIZE );

  /* receive temporary vector structure to check */
  code = MPI_Recv( &vtmp, sizeof( vtmp ), MPI_BYTE, source, MPIU4Vector, comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* make sure that the vector lengths agree */
  if ( vector->length != vtmp.length )
  {
    ABORT( status, COMMH_ESZMM, COMMH_MSGESZMM );
  }

  /* receive the vector data */
  code = MPI_Recv( vector->data, vector->length * sizeof( *vector->data ),
      MPI_BYTE, source, MPIU4VectorData, comm, &stat );

  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}






















/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendUINT8Vector ( LALStatus *status, UINT8Vector *vector, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendUINT8Vector", SENDRECVC );

  ASSERT( vector,         status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->data,   status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->length, status, COMMH_ESIZE, COMMH_MSGESIZE );

  /* send the vector structure */
  code = MPI_Send( vector, sizeof( *vector ), MPI_BYTE, dest, MPIU8Vector, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send the vector data */
  code = MPI_Send( vector->data, vector->length * sizeof( *vector->data ),
      MPI_BYTE, dest, MPIU8VectorData, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvUINT8Vector ( LALStatus *status, UINT8Vector *vector, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  UINT8Vector vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvUINT8Vector", SENDRECVC );

  ASSERT( vector,         status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->data,   status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->length, status, COMMH_ESIZE, COMMH_MSGESIZE );

  /* receive temporary vector structure to check */
  code = MPI_Recv( &vtmp, sizeof( vtmp ), MPI_BYTE, source, MPIU8Vector, comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* make sure that the vector lengths agree */
  if ( vector->length != vtmp.length )
  {
    ABORT( status, COMMH_ESZMM, COMMH_MSGESZMM );
  }

  /* receive the vector data */
  code = MPI_Recv( vector->data, vector->length * sizeof( *vector->data ),
      MPI_BYTE, source, MPIU8VectorData, comm, &stat );

  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}






















/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendCHARVector ( LALStatus *status, CHARVector *vector, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendCHARVector", SENDRECVC );

  ASSERT( vector,         status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->data,   status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->length, status, COMMH_ESIZE, COMMH_MSGESIZE );

  /* send the vector structure */
  code = MPI_Send( vector, sizeof( *vector ), MPI_BYTE, dest, MPICHARVector, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send the vector data */
  code = MPI_Send( vector->data, vector->length * sizeof( *vector->data ),
      MPI_BYTE, dest, MPICHARVectorData, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvCHARVector ( LALStatus *status, CHARVector *vector, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  CHARVector vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvCHARVector", SENDRECVC );

  ASSERT( vector,         status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->data,   status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( vector->length, status, COMMH_ESIZE, COMMH_MSGESIZE );

  /* receive temporary vector structure to check */
  code = MPI_Recv( &vtmp, sizeof( vtmp ), MPI_BYTE, source, MPICHARVector, comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* make sure that the vector lengths agree */
  if ( vector->length != vtmp.length )
  {
    ABORT( status, COMMH_ESZMM, COMMH_MSGESZMM );
  }

  /* receive the vector data */
  code = MPI_Recv( vector->data, vector->length * sizeof( *vector->data ),
      MPI_BYTE, source, MPICHARVectorData, comm, &stat );

  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  RETURN( status );
}






























/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendCOMPLEX16TimeSeries ( LALStatus *status, COMPLEX16TimeSeries *series, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendCOMPLEX16TimeSeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the series structure */
  code = MPI_Send( series, sizeof( *series ), MPI_BYTE, dest, MPIZTimeSeries, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send data vector */
  LALMPISendCOMPLEX16Vector ( status->statusPtr, series->data, dest, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvCOMPLEX16TimeSeries ( LALStatus *status, COMPLEX16TimeSeries *series, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  COMPLEX16Vector *vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvCOMPLEX16TimeSeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* receive time series structure */
  vtmp = series->data;
  code = MPI_Recv( series, sizeof( *series ), MPI_BYTE, source, MPIZTimeSeries,
      comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }
  series->data = vtmp;

  /* receive data vector */
  LALMPIRecvCOMPLEX16Vector ( status->statusPtr, series->data, source, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



























/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendCOMPLEX8TimeSeries ( LALStatus *status, COMPLEX8TimeSeries *series, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendCOMPLEX8TimeSeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the series structure */
  code = MPI_Send( series, sizeof( *series ), MPI_BYTE, dest, MPICTimeSeries, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send data vector */
  LALMPISendCOMPLEX8Vector ( status->statusPtr, series->data, dest, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvCOMPLEX8TimeSeries ( LALStatus *status, COMPLEX8TimeSeries *series, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  COMPLEX8Vector *vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvCOMPLEX8TimeSeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* receive time series structure */
  vtmp = series->data;
  code = MPI_Recv( series, sizeof( *series ), MPI_BYTE, source, MPICTimeSeries,
      comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }
  series->data = vtmp;

  /* receive data vector */
  LALMPIRecvCOMPLEX8Vector ( status->statusPtr, series->data, source, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



























/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendREAL8TimeSeries ( LALStatus *status, REAL8TimeSeries *series, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendREAL8TimeSeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the series structure */
  code = MPI_Send( series, sizeof( *series ), MPI_BYTE, dest, MPIDTimeSeries, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send data vector */
  LALMPISendREAL8Vector ( status->statusPtr, series->data, dest, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvREAL8TimeSeries ( LALStatus *status, REAL8TimeSeries *series, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  REAL8Vector *vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvREAL8TimeSeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* receive time series structure */
  vtmp = series->data;
  code = MPI_Recv( series, sizeof( *series ), MPI_BYTE, source, MPIDTimeSeries,
      comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }
  series->data = vtmp;

  /* receive data vector */
  LALMPIRecvREAL8Vector ( status->statusPtr, series->data, source, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



























/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendREAL4TimeSeries ( LALStatus *status, REAL4TimeSeries *series, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendREAL4TimeSeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the series structure */
  code = MPI_Send( series, sizeof( *series ), MPI_BYTE, dest, MPISTimeSeries, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send data vector */
  LALMPISendREAL4Vector ( status->statusPtr, series->data, dest, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvREAL4TimeSeries ( LALStatus *status, REAL4TimeSeries *series, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  REAL4Vector *vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvREAL4TimeSeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* receive time series structure */
  vtmp = series->data;
  code = MPI_Recv( series, sizeof( *series ), MPI_BYTE, source, MPISTimeSeries,
      comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }
  series->data = vtmp;

  /* receive data vector */
  LALMPIRecvREAL4Vector ( status->statusPtr, series->data, source, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



























/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendINT2TimeSeries ( LALStatus *status, INT2TimeSeries *series, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendINT2TimeSeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the series structure */
  code = MPI_Send( series, sizeof( *series ), MPI_BYTE, dest, MPII2TimeSeries, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send data vector */
  LALMPISendINT2Vector ( status->statusPtr, series->data, dest, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvINT2TimeSeries ( LALStatus *status, INT2TimeSeries *series, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  INT2Vector *vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvINT2TimeSeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* receive time series structure */
  vtmp = series->data;
  code = MPI_Recv( series, sizeof( *series ), MPI_BYTE, source, MPII2TimeSeries,
      comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }
  series->data = vtmp;

  /* receive data vector */
  LALMPIRecvINT2Vector ( status->statusPtr, series->data, source, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



























/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendINT4TimeSeries ( LALStatus *status, INT4TimeSeries *series, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendINT4TimeSeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the series structure */
  code = MPI_Send( series, sizeof( *series ), MPI_BYTE, dest, MPII4TimeSeries, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send data vector */
  LALMPISendINT4Vector ( status->statusPtr, series->data, dest, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvINT4TimeSeries ( LALStatus *status, INT4TimeSeries *series, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  INT4Vector *vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvINT4TimeSeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* receive time series structure */
  vtmp = series->data;
  code = MPI_Recv( series, sizeof( *series ), MPI_BYTE, source, MPII4TimeSeries,
      comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }
  series->data = vtmp;

  /* receive data vector */
  LALMPIRecvINT4Vector ( status->statusPtr, series->data, source, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



























/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendINT8TimeSeries ( LALStatus *status, INT8TimeSeries *series, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendINT8TimeSeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the series structure */
  code = MPI_Send( series, sizeof( *series ), MPI_BYTE, dest, MPII8TimeSeries, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send data vector */
  LALMPISendINT8Vector ( status->statusPtr, series->data, dest, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvINT8TimeSeries ( LALStatus *status, INT8TimeSeries *series, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  INT8Vector *vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvINT8TimeSeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* receive time series structure */
  vtmp = series->data;
  code = MPI_Recv( series, sizeof( *series ), MPI_BYTE, source, MPII8TimeSeries,
      comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }
  series->data = vtmp;

  /* receive data vector */
  LALMPIRecvINT8Vector ( status->statusPtr, series->data, source, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



























/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendUINT2TimeSeries ( LALStatus *status, UINT2TimeSeries *series, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendUINT2TimeSeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the series structure */
  code = MPI_Send( series, sizeof( *series ), MPI_BYTE, dest, MPIU2TimeSeries, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send data vector */
  LALMPISendUINT2Vector ( status->statusPtr, series->data, dest, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvUINT2TimeSeries ( LALStatus *status, UINT2TimeSeries *series, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  UINT2Vector *vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvUINT2TimeSeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* receive time series structure */
  vtmp = series->data;
  code = MPI_Recv( series, sizeof( *series ), MPI_BYTE, source, MPIU2TimeSeries,
      comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }
  series->data = vtmp;

  /* receive data vector */
  LALMPIRecvUINT2Vector ( status->statusPtr, series->data, source, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



























/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendUINT4TimeSeries ( LALStatus *status, UINT4TimeSeries *series, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendUINT4TimeSeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the series structure */
  code = MPI_Send( series, sizeof( *series ), MPI_BYTE, dest, MPIU4TimeSeries, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send data vector */
  LALMPISendUINT4Vector ( status->statusPtr, series->data, dest, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvUINT4TimeSeries ( LALStatus *status, UINT4TimeSeries *series, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  UINT4Vector *vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvUINT4TimeSeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* receive time series structure */
  vtmp = series->data;
  code = MPI_Recv( series, sizeof( *series ), MPI_BYTE, source, MPIU4TimeSeries,
      comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }
  series->data = vtmp;

  /* receive data vector */
  LALMPIRecvUINT4Vector ( status->statusPtr, series->data, source, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



























/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendUINT8TimeSeries ( LALStatus *status, UINT8TimeSeries *series, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendUINT8TimeSeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the series structure */
  code = MPI_Send( series, sizeof( *series ), MPI_BYTE, dest, MPIU8TimeSeries, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send data vector */
  LALMPISendUINT8Vector ( status->statusPtr, series->data, dest, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvUINT8TimeSeries ( LALStatus *status, UINT8TimeSeries *series, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  UINT8Vector *vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvUINT8TimeSeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* receive time series structure */
  vtmp = series->data;
  code = MPI_Recv( series, sizeof( *series ), MPI_BYTE, source, MPIU8TimeSeries,
      comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }
  series->data = vtmp;

  /* receive data vector */
  LALMPIRecvUINT8Vector ( status->statusPtr, series->data, source, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}






























/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendCOMPLEX16FrequencySeries ( LALStatus *status, COMPLEX16FrequencySeries *series, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendCOMPLEX16FrequencySeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the series structure */
  code = MPI_Send( series, sizeof( *series ), MPI_BYTE, dest, MPIZTimeSeries, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send data vector */
  LALMPISendCOMPLEX16Vector ( status->statusPtr, series->data, dest, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvCOMPLEX16FrequencySeries ( LALStatus *status, COMPLEX16FrequencySeries *series, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  COMPLEX16Vector *vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvCOMPLEX16FrequencySeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* receive time series structure */
  vtmp = series->data;
  code = MPI_Recv( series, sizeof( *series ), MPI_BYTE, source, MPIZTimeSeries,
      comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }
  series->data = vtmp;

  /* receive data vector */
  LALMPIRecvCOMPLEX16Vector ( status->statusPtr, series->data, source, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



























/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendCOMPLEX8FrequencySeries ( LALStatus *status, COMPLEX8FrequencySeries *series, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendCOMPLEX8FrequencySeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the series structure */
  code = MPI_Send( series, sizeof( *series ), MPI_BYTE, dest, MPICTimeSeries, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send data vector */
  LALMPISendCOMPLEX8Vector ( status->statusPtr, series->data, dest, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvCOMPLEX8FrequencySeries ( LALStatus *status, COMPLEX8FrequencySeries *series, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  COMPLEX8Vector *vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvCOMPLEX8FrequencySeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* receive time series structure */
  vtmp = series->data;
  code = MPI_Recv( series, sizeof( *series ), MPI_BYTE, source, MPICTimeSeries,
      comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }
  series->data = vtmp;

  /* receive data vector */
  LALMPIRecvCOMPLEX8Vector ( status->statusPtr, series->data, source, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



























/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendREAL8FrequencySeries ( LALStatus *status, REAL8FrequencySeries *series, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendREAL8FrequencySeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the series structure */
  code = MPI_Send( series, sizeof( *series ), MPI_BYTE, dest, MPIDTimeSeries, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send data vector */
  LALMPISendREAL8Vector ( status->statusPtr, series->data, dest, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvREAL8FrequencySeries ( LALStatus *status, REAL8FrequencySeries *series, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  REAL8Vector *vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvREAL8FrequencySeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* receive time series structure */
  vtmp = series->data;
  code = MPI_Recv( series, sizeof( *series ), MPI_BYTE, source, MPIDTimeSeries,
      comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }
  series->data = vtmp;

  /* receive data vector */
  LALMPIRecvREAL8Vector ( status->statusPtr, series->data, source, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



























/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendREAL4FrequencySeries ( LALStatus *status, REAL4FrequencySeries *series, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendREAL4FrequencySeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the series structure */
  code = MPI_Send( series, sizeof( *series ), MPI_BYTE, dest, MPISTimeSeries, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send data vector */
  LALMPISendREAL4Vector ( status->statusPtr, series->data, dest, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvREAL4FrequencySeries ( LALStatus *status, REAL4FrequencySeries *series, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  REAL4Vector *vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvREAL4FrequencySeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* receive time series structure */
  vtmp = series->data;
  code = MPI_Recv( series, sizeof( *series ), MPI_BYTE, source, MPISTimeSeries,
      comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }
  series->data = vtmp;

  /* receive data vector */
  LALMPIRecvREAL4Vector ( status->statusPtr, series->data, source, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



























/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendINT2FrequencySeries ( LALStatus *status, INT2FrequencySeries *series, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendINT2FrequencySeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the series structure */
  code = MPI_Send( series, sizeof( *series ), MPI_BYTE, dest, MPII2TimeSeries, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send data vector */
  LALMPISendINT2Vector ( status->statusPtr, series->data, dest, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvINT2FrequencySeries ( LALStatus *status, INT2FrequencySeries *series, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  INT2Vector *vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvINT2FrequencySeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* receive time series structure */
  vtmp = series->data;
  code = MPI_Recv( series, sizeof( *series ), MPI_BYTE, source, MPII2TimeSeries,
      comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }
  series->data = vtmp;

  /* receive data vector */
  LALMPIRecvINT2Vector ( status->statusPtr, series->data, source, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



























/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendINT4FrequencySeries ( LALStatus *status, INT4FrequencySeries *series, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendINT4FrequencySeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the series structure */
  code = MPI_Send( series, sizeof( *series ), MPI_BYTE, dest, MPII4TimeSeries, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send data vector */
  LALMPISendINT4Vector ( status->statusPtr, series->data, dest, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvINT4FrequencySeries ( LALStatus *status, INT4FrequencySeries *series, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  INT4Vector *vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvINT4FrequencySeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* receive time series structure */
  vtmp = series->data;
  code = MPI_Recv( series, sizeof( *series ), MPI_BYTE, source, MPII4TimeSeries,
      comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }
  series->data = vtmp;

  /* receive data vector */
  LALMPIRecvINT4Vector ( status->statusPtr, series->data, source, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



























/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendINT8FrequencySeries ( LALStatus *status, INT8FrequencySeries *series, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendINT8FrequencySeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the series structure */
  code = MPI_Send( series, sizeof( *series ), MPI_BYTE, dest, MPII8TimeSeries, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send data vector */
  LALMPISendINT8Vector ( status->statusPtr, series->data, dest, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvINT8FrequencySeries ( LALStatus *status, INT8FrequencySeries *series, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  INT8Vector *vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvINT8FrequencySeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* receive time series structure */
  vtmp = series->data;
  code = MPI_Recv( series, sizeof( *series ), MPI_BYTE, source, MPII8TimeSeries,
      comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }
  series->data = vtmp;

  /* receive data vector */
  LALMPIRecvINT8Vector ( status->statusPtr, series->data, source, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



























/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendUINT2FrequencySeries ( LALStatus *status, UINT2FrequencySeries *series, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendUINT2FrequencySeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the series structure */
  code = MPI_Send( series, sizeof( *series ), MPI_BYTE, dest, MPIU2TimeSeries, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send data vector */
  LALMPISendUINT2Vector ( status->statusPtr, series->data, dest, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvUINT2FrequencySeries ( LALStatus *status, UINT2FrequencySeries *series, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  UINT2Vector *vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvUINT2FrequencySeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* receive time series structure */
  vtmp = series->data;
  code = MPI_Recv( series, sizeof( *series ), MPI_BYTE, source, MPIU2TimeSeries,
      comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }
  series->data = vtmp;

  /* receive data vector */
  LALMPIRecvUINT2Vector ( status->statusPtr, series->data, source, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



























/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendUINT4FrequencySeries ( LALStatus *status, UINT4FrequencySeries *series, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendUINT4FrequencySeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the series structure */
  code = MPI_Send( series, sizeof( *series ), MPI_BYTE, dest, MPIU4TimeSeries, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send data vector */
  LALMPISendUINT4Vector ( status->statusPtr, series->data, dest, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvUINT4FrequencySeries ( LALStatus *status, UINT4FrequencySeries *series, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  UINT4Vector *vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvUINT4FrequencySeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* receive time series structure */
  vtmp = series->data;
  code = MPI_Recv( series, sizeof( *series ), MPI_BYTE, source, MPIU4TimeSeries,
      comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }
  series->data = vtmp;

  /* receive data vector */
  LALMPIRecvUINT4Vector ( status->statusPtr, series->data, source, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



























/* <lalVerbatim file="SendRecvCP"> */
void LALMPISendUINT8FrequencySeries ( LALStatus *status, UINT8FrequencySeries *series, INT4 dest, MPI_Comm comm )
{ /* </lalVerbatim> */
  INT4 code;

  INITSTATUS( status, "LALMPISendUINT8FrequencySeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* send the series structure */
  code = MPI_Send( series, sizeof( *series ), MPI_BYTE, dest, MPIU8TimeSeries, comm );
  if ( code != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }

  /* send data vector */
  LALMPISendUINT8Vector ( status->statusPtr, series->data, dest, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

/* <lalVerbatim file="SendRecvCP"> */
void LALMPIRecvUINT8FrequencySeries ( LALStatus *status, UINT8FrequencySeries *series, INT4 source, MPI_Comm comm )
{ /* </lalVerbatim> */
  MPI_Status stat;
  UINT8Vector *vtmp;
  INT4 code;

  INITSTATUS( status, "LALMPIRecvUINT8FrequencySeries", SENDRECVC );
  ATTATCHSTATUSPTR( status );

  ASSERT( series,       status, COMMH_ENULL, COMMH_MSGENULL );
  ASSERT( series->data, status, COMMH_ENULL, COMMH_MSGENULL );

  /* receive time series structure */
  vtmp = series->data;
  code = MPI_Recv( series, sizeof( *series ), MPI_BYTE, source, MPIU8TimeSeries,
      comm, &stat );
  if ( code != MPI_SUCCESS || stat.MPI_ERROR != MPI_SUCCESS )
  {
    ABORT( status, COMMH_EMPIE, COMMH_MSGEMPIE );
  }
  series->data = vtmp;

  /* receive data vector */
  LALMPIRecvUINT8Vector ( status->statusPtr, series->data, source, comm );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

